@@ -54,7 +54,8 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
-MakefileGenerator::MakefileGenerator(QMakeProject *p) : init_already(FALSE), moc_aware(FALSE), project(p)
+MakefileGenerator::MakefileGenerator(QMakeProject *p) : init_opath_already(FALSE),
+							init_already(FALSE), moc_aware(FALSE), project(p)
 {
 }
 
@@ -92,7 +93,7 @@ MakefileGenerator::generateMocList(QString fn_target)
     close(file);
 
     bool ignore_qobject = FALSE;
-    int line_count = 0;
+    int line_count = 1;
  /* qmake ignore Q_OBJECT */
 #define COMP_LEN 8 //strlen("Q_OBJECT")
 #define OBJ_LEN 8 //strlen("Q_OBJECT")
@@ -216,17 +217,21 @@ MakefileGenerator::generateDependencies(QPtrList<MakefileDependDir> &dirs, QStri
 	return TRUE;
 
     fn = Option::fixPathToLocalOS(fn, FALSE);
-
-    QString fndir;
+    QString fix_env_fn = Option::fixPathToLocalOS(fn);
+    int file = open(fix_env_fn.latin1(), O_RDONLY);
+    if(file == -1) 
+	return FALSE;
+    QString fndir, fix_env_fndir;
     int dl = fn.findRev(Option::dir_sep);
     if(dl != -1) 
 	fndir = fn.left(dl+1);
+    dl = fix_env_fn.findRev(Option::dir_sep);
+    if(dl != -1)
+	fix_env_fndir = fix_env_fn.left(dl + 1);
 
-    int file = open(fn.latin1(), O_RDONLY);
-    if(file == -1)
-	return FALSE;
 
     struct stat fst;
+    int line_count = 1;
     if(fstat(file, &fst))
 	return FALSE; //shouldn't happen
     char *big_buffer = gimme_buffer(fst.st_size);
@@ -258,6 +263,8 @@ MakefileGenerator::generateDependencies(QPtrList<MakefileDependDir> &dirs, QStri
 				    x += 2;
 				    break;
 				}
+			    } else if(*(big_buffer + x) == '\n') {
+				line_count++;
 			    }
 			}
 		    }
@@ -265,13 +272,11 @@ MakefileGenerator::generateDependencies(QPtrList<MakefileDependDir> &dirs, QStri
 	    }
 	    if(*(big_buffer + x) == '#') {
 		x++;
-		// skip spaces and tabs between the hash and text, so we can handle
-		// #  include
-		while(x < total_size_read &&
+		while(x < total_size_read && //Skip spaces after hash
 		      (*(big_buffer+x) == ' ' || *(big_buffer+x) == '\t'))
 		    x++;
 		if(total_size_read >= x + 8 && !strncmp(big_buffer + x, "include ", 8)) {
-		    for(x+=8;
+		    for(x+=8; //skip spaces after keyword
 			x < total_size_read && (*(big_buffer+x) == ' ' || *(big_buffer+x) == '\t');
 			x++);
 		    char term = *(big_buffer + x);
@@ -285,8 +290,26 @@ MakefileGenerator::generateDependencies(QPtrList<MakefileDependDir> &dirs, QStri
 		    int inc_len;
 		    for(inc_len = 0; *(big_buffer + x + inc_len) != term; inc_len++);
 		    *(big_buffer + x + inc_len) = '\0';
-
 		    inc = big_buffer + x;
+		} else if(total_size_read >= x + 14 && !strncmp(big_buffer + x,  "qmake_warning ", 14)) {
+		    for(x+=14; //skip spaces after keyword
+			x < total_size_read && (*(big_buffer+x) == ' ' || *(big_buffer+x) == '\t');
+			x++);
+		    char term = '\n';
+		    if(*(big_buffer + x) == '"')
+			term = '"';
+		    if(*(big_buffer + x) == '\'')
+			term = '\'';
+		    if(term != '\n')
+			x++;
+
+		    int msg_len;
+		    for(msg_len = 0; *(big_buffer + x + msg_len) != term; msg_len++);
+		    *(big_buffer + x + msg_len) = '\0';
+		    QString msg = big_buffer + x;
+		    debug_msg(0, "%s:%d qmake_warning -- %s", fix_env_fn.latin1(), 
+			      line_count, msg.latin1());
+		    *(big_buffer + x + msg_len) = term; //put it back
 		}
 	    }
 	} else if(ftype == UI_FILE) {
@@ -307,9 +330,10 @@ MakefileGenerator::generateDependencies(QPtrList<MakefileDependDir> &dirs, QStri
 		}
 	    }
 	}
-	debug_msg(5, "Found dependency from %s to %s", fn.latin1(), inc.latin1());
 
 	if(!inc.isEmpty()) {
+	    debug_msg(5, "%s:%d Found dependency to %s", fix_env_fn.latin1(), 
+		      line_count, inc.latin1());
 	    if(!project->isEmpty("SKIP_DEPENDS")) {
 		bool found = FALSE;
 		QStringList &nodeplist = project->values("SKIP_DEPENDS");
@@ -326,7 +350,7 @@ MakefileGenerator::generateDependencies(QPtrList<MakefileDependDir> &dirs, QStri
 	    }
 
 	    QString fqn;
-	    if(project->isEmpty("QMAKE_ABSOLUTE_SOURCE_PATH") && !stat(fndir + inc, &fst)) {
+	    if(project->isEmpty("QMAKE_ABSOLUTE_SOURCE_PATH") && !stat(fix_env_fndir + inc, &fst)) {
 		fqn = fndir + inc;
 	    } else {
 		if((Option::target_mode == Option::TARG_MAC9_MODE && inc.find(':')) ||
@@ -358,10 +382,12 @@ MakefileGenerator::generateDependencies(QPtrList<MakefileDependDir> &dirs, QStri
 			QStringList uil = project->variables()["FORMS"];
 			for(QStringList::Iterator it = uil.begin(); it != uil.end(); ++it) {
 			    QString s = (*it);
-			    if(!project->isEmpty("UI_DIR")) {
 				int d = s.findRev(Option::dir_sep) + 1;
+			    if(!project->isEmpty("UI_DIR")) 
 				s = project->first("UI_DIR") + s.right(s.length()-d);
-			    }
+			    else if(!project->isEmpty("UI_HEADERS_DIR")) 
+				s = project->first("UI_HEADERS_DIR") + s.right(s.length()-d);
+				
 			    if(s.find(QRegExp(uip)) != -1) {
 				fqn = s.left(s.length()-3) + inc.right(inc.length()-extn);
 				break;
@@ -403,6 +429,7 @@ MakefileGenerator::generateDependencies(QPtrList<MakefileDependDir> &dirs, QStri
 	}
 	//read past new line now..
 	for( ; x < total_size_read && (*(big_buffer + x) != '\n'); x++);
+        line_count++;
     }
 
     if(recurse) {
@@ -419,14 +446,12 @@ MakefileGenerator::generateDependencies(QPtrList<MakefileDependDir> &dirs, QStri
 }
 
 void
-MakefileGenerator::init()
+MakefileGenerator::initOutPaths()
 {
-    if(init_already)
+    if(init_opath_already)
 	return;
-    init_already = TRUE;
-
+    init_opath_already = TRUE;
     QMap<QString, QStringList> &v = project->variables();
-    { //opaths
 	if(!v.contains("QMAKE_ABSOLUTE_SOURCE_PATH")) {
 	    if(Option::mkfile::do_cache && !Option::mkfile::cachefile.isEmpty() &&
 	       v.contains("QMAKE_ABSOLUTE_SOURCE_ROOT")) {
@@ -452,7 +477,9 @@ MakefileGenerator::init()
 		v["QMAKE_ABSOLUTE_SOURCE_PATH"].clear();
 	}
 	QString currentDir = QDir::currentDirPath();
-	QString dirs[] = { QString("OBJECTS_DIR"), QString("MOC_DIR"), QString("UI_DIR"), QString("DESTDIR"), QString("SUBLIBS_DIR"), QString::null };
+	QString dirs[] = { QString("OBJECTS_DIR"), QString("MOC_DIR"), QString("UI_HEADERS_DIR"), 
+			   QString("UI_SOURCES_DIR"), QString("UI_DIR"), QString("DESTDIR"), 
+			   QString("SUBLIBS_DIR"), QString::null };
 	for(int x = 0; dirs[x] != QString::null; x++) {
 	    if ( !v[dirs[x]].isEmpty() ) {
 		{
@@ -467,23 +494,55 @@ MakefileGenerator::init()
 		path = Option::fixPathToTargetOS(path);
 
 		QDir d;
-		if ( !QDir::isRelativePath( path ) )
-		    d.cd( path.left( 2 ) );
-		if(path.left(1) == Option::dir_sep)
+		if(path.left(1) == Option::dir_sep) {
 		    d.cd(Option::dir_sep);
-
-		QStringList subs = QStringList::split(Option::dir_sep, path);
-		for(QStringList::Iterator subit = subs.begin(); subit != subs.end(); ++subit) {
-		    if(!d.cd(*subit)) {
-			d.mkdir((*subit));
-			d.cd((*subit));
+		    path = path.right( path.length() - 1 );
+		}
+#ifdef Q_WS_WIN
+		bool driveExists = TRUE;
+		if ( !QDir::isRelativePath( path ) ) {
+		    if ( QFile::exists( path.left( 3 ) ) ) {
+			d.cd( path.left( 3 ) );
+			path = path.right( path.length() - 3 );
+		    } else {
+			warn_msg(WarnLogic, "%s: Cannot access drive '%s' (%s)", dirs[x].latin1(), 
+			    path.left( 3 ).latin1(), path.latin1() );
+			driveExists = FALSE;
 		    }
 		}
+		if ( driveExists ) {
+#endif
+		    QStringList subs = QStringList::split(Option::dir_sep, path);
+		    for(QStringList::Iterator subit = subs.begin(); subit != subs.end(); ++subit) {
+			if(!d.cd(*subit)) {
+			    d.mkdir((*subit));
+			    if ( d.exists( (*subit) ) )
+				d.cd((*subit));
+			    else {
+				warn_msg(WarnLogic, "%s: Cannot access directory '%s' (%s)", dirs[x].latin1(), 
+				    (*subit).latin1(), path.latin1() );
+				break;
+			    }
+			}
+		    }
+#ifdef Q_WS_WIN
+		}
+#endif
 	    }
 	}
 	QDir::current().cd( currentDir );
-    }
 
+}
+
+void
+MakefileGenerator::init()
+{
+    initOutPaths();
+    if(init_already)
+	return;
+    init_already = TRUE;
+
+    QMap<QString, QStringList> &v = project->variables();
     QString paths[] = { QString("SOURCES"), QString("FORMS"), QString("YACCSOURCES"), QString("INCLUDEPATH"),
 			    QString("HEADERS"), QString("HEADERS_ORIG"),
 			    QString("LEXSOURCES"), QString("QMAKE_INTERNAL_INCLUDED_FILES"), QString::null };
@@ -722,7 +781,9 @@ MakefileGenerator::init()
 		       << "QMAKE_CACHE_VERSION = " << qmake_version() << "\n"
 		       << "QMAKE_ABSOLUTE_SOURCE_PATH = " << var("QMAKE_ABSOLUTE_SOURCE_PATH") << "\n"
 		       << "MOC_DIR = " << var("MOC_DIR") << "\n"
-		       << "UI_DIR = " <<  var("UI_DIR") << "\n";
+		       << "UI_DIR = " <<  var("UI_DIR") << "\n"
+		       << "UI_HEADERS_DIR = " <<  var("UI_HEADERS_DIR") << "\n"
+		       << "UI_SOURCES_DIR = " <<  var("UI_SOURCES_DIR") << "\n";
 		cachet << "[depend]" << endl;
 		for(QMap<QString, QStringList>::Iterator it = depends.begin(); 
 		    it != depends.end(); ++it) 
@@ -821,28 +882,42 @@ MakefileGenerator::init()
     {
 	if(!project->isEmpty("UI_DIR"))
 	    project->variables()["INCLUDEPATH"].append(project->first("UI_DIR"));
+	else if(!project->isEmpty("UI_HEADERS_DIR"))
+	    project->variables()["INCLUDEPATH"].append(project->first("UI_HEADERS_DIR"));
 	QStringList &decls = v["UICDECLS"], &impls = v["UICIMPLS"];
 	QStringList &l = v["FORMS"];
 	for(QStringList::Iterator it = l.begin(); it != l.end(); ++it) {
-	    QFileInfo fi(Option::fixPathToLocalOS((*it)));
 	    QString impl, decl;
+	    QFileInfo fi(Option::fixPathToLocalOS((*it)));
 	    if ( !project->isEmpty("UI_DIR") ) {
-		impl = project->first("UI_DIR") + fi.baseName(TRUE) + Option::cpp_ext.first();
-		decl = project->first("UI_DIR") + fi.baseName(TRUE) + Option::h_ext.first();
-
+		impl = decl = project->first("UI_DIR");
 		QString d = fi.dirPath();
 		if( d == ".")
 		    d = QDir::currentDirPath();
 		fileFixify(d);
 		if( !project->variables()["INCLUDEPATH"].contains(d))
 		    project->variables()["INCLUDEPATH"].append(d);
-	    } else if ( fi.dirPath() == "." ) {
-	    	impl = fi.baseName(TRUE) + Option::cpp_ext.first();
-                decl = fi.baseName(TRUE) + Option::h_ext.first();
 	    } else {
-	    	impl = fi.dirPath() + Option::dir_sep + fi.baseName(TRUE) + Option::cpp_ext.first();
-                decl = fi.dirPath() + Option::dir_sep + fi.baseName(TRUE) + Option::h_ext.first();
+		if(decl.isEmpty() && !project->isEmpty("UI_HEADERS_DIR")) {
+		    decl = project->first("UI_HEADERS_DIR");
+		    QString d = fi.dirPath();
+		    if( d == ".")
+			d = QDir::currentDirPath();
+		    fileFixify(d);
+		    if( !project->variables()["INCLUDEPATH"].contains(d))
+			project->variables()["INCLUDEPATH"].append(d);
+		}
+		if(impl.isEmpty() && !project->isEmpty("UI_SOURCES_DIR")) 
+		    impl = project->first("UI_SOURCES_DIR");
+		if(fi.dirPath() != ".") {
+		    if(impl.isEmpty())
+			impl = fi.dirPath() + Option::dir_sep;
+		    if(decl.isEmpty())
+			decl = fi.dirPath() + Option::dir_sep;
+		}
 	    }
+	    impl += fi.baseName(TRUE) + Option::cpp_ext.first(), 
+	    decl += fi.baseName(TRUE) + Option::h_ext.first();
 	    logicWarn(impl, "SOURCES");
 	    logicWarn(decl, "HEADERS");
 	    decls.append(decl);
@@ -868,10 +943,11 @@ MakefileGenerator::init()
 	    v["QMAKE_IMAGE_COLLECTION"].append("qmake_image_collection" + Option::cpp_ext.first());
 	QString imgfile = project->first("QMAKE_IMAGE_COLLECTION");
 	Option::fixPathToTargetOS(imgfile);
-	if(!project->isEmpty("UI_DIR")) {
+	if(!project->isEmpty("UI_DIR") || !project->isEmpty("UI_SOURCES_DIR")) {
 	    if(imgfile.find(Option::dir_sep) != -1)
 		imgfile = imgfile.right(imgfile.findRev(Option::dir_sep) + 1);
-	    imgfile.prepend(project->first("UI_DIR"));
+	    imgfile.prepend( (project->isEmpty("UI_DIR") ? project->first("UI_SOURCES_DIR") : 
+			    project->first("UI_DIR")) );
 	    v["QMAKE_IMAGE_COLLECTION"] = QStringList(imgfile);
 	}
 	logicWarn(imgfile, "SOURCES");
@@ -1155,10 +1231,15 @@ MakefileGenerator::writeUicSrc(QTextStream &t, const QString &ui)
 	    decl = tmp.replace(QRegExp("\\" + Option::ui_ext + "$"), Option::h_ext.first());
 	    tmp = (*it);
 	    impl = tmp.replace(QRegExp("\\" + Option::ui_ext + "$"), Option::cpp_ext.first());
-	    if(!project->isEmpty("UI_DIR")) {
 		int dlen = (*it).findRev(Option::dir_sep) + 1;
+	    if(!project->isEmpty("UI_DIR")) {
 		decl = project->first("UI_DIR") + decl.right(decl.length() - dlen);
 		impl = project->first("UI_DIR") + impl.right(impl.length() - dlen);
+	    } else {
+		if(!project->isEmpty("UI_HEADERS_DIR"))
+		    decl = project->first("UI_HEADERS_DIR") + decl.right(decl.length() - dlen);
+		if(!project->isEmpty("UI_SOURCES_DIR"))
+		    impl = project->first("UI_SOURCES_DIR") + impl.right(impl.length() - dlen);
 	    }
 	}
 	t << decl << ": " << (*it) << " " << deps << "\n\t"
@@ -1282,7 +1363,7 @@ MakefileGenerator::writeImageObj(QTextStream &t, const QString &obj)
         QString src(project->first("QMAKE_IMAGE_COLLECTION"));
 	t << (*oit) << ": " << src;
 	if ( !project->isEmpty("OBJECTS_DIR") || !project->isEmpty("UI_DIR") ||
-	     project->isEmpty("QMAKE_RUN_CXX_IMP")) {
+	     !project->isEmpty("UI_SOURCES_DIR") || project->isEmpty("QMAKE_RUN_CXX_IMP")) {
 	    QString p = var("QMAKE_RUN_CXX");
 	    p.replace( regexpSrc, src);
 	    p.replace( regexpObj, (*oit));
@@ -1314,15 +1395,24 @@ MakefileGenerator::writeIdlSrc(QTextStream &t, const QString &src)
 	input = varGlue( "_ACTIVEQT", "", " ", "" );
 	inputList = "_ACTIVEQT";
     }
+    bool fromIDL = FALSE;
+    QString inputFile;
+    if ( project->variables()[inputList].count() == 1 ) {
+	inputFile = project->variables()[inputList].first();
+	fromIDL = inputFile.right( 4 ).lower() == ".idl";
+    }
     for(QStringList::Iterator it = l.begin(); it != l.end(); ++it) {
 	QString file = *it;
-	if ( file.right( 4 ) != ".res" )
+	if ( file.right( 4 ).lower() != ".res" )
 	    continue;
 	QString idlfile = file.left( file.length()-3) += "idl";
+	if ( fromIDL )
+	    idlfile = inputFile;
 	QString rcfile = file.left( file.length()-3) += "rc";
-	t << file << ": " << project->variables()[inputList].join(" \\\n\t\t") << "\n\t"
-	    << "$(IDC) " << input << " -o " << idlfile << " -rc " << rcfile << "\n\t"
-	    << "midl " << idlfile << " /tlb " << file.left( file.length()-3) << "tlb"
+	t << file << ": " << project->variables()[inputList].join(" \\\n\t\t") << "\n\t";
+	    if ( !fromIDL )
+		t << "$(IDC) " << input << " -o " << idlfile << " -rc " << rcfile << "\n\t";
+	    t << "midl " << idlfile << " /tlb " << file.left( file.length()-3) << "tlb"
 	    << " /iid tmp\\iid_i.c /dlldata tmp\\dlldata.c /cstub tmp\\cstub.c /header tmp\\cstub.h /proxy tmp\\proxy.c /sstub tmp\\sstub.c\n\t"
 	    << var("QMAKE_RC") << " " << rcfile << endl << endl;
 	break;
@@ -1342,7 +1432,7 @@ MakefileGenerator::writeInstalls(QTextStream &t, const QString &installs)
 	}
 
 	bool do_default = TRUE;
-	QString target, dst=project->variables()[pvar].first();
+	QString target, dst=QString("$(INSTALL_ROOT)") + project->variables()[pvar].first();
 	if(dst.right(1) != Option::dir_sep)
 	    dst += Option::dir_sep;
  	QStringList tmp, &uninst = project->variables()[(*it) + ".uninstall"];
@@ -1466,10 +1556,14 @@ MakefileGenerator::createObjectList(const QString &var)
     for(QStringList::Iterator it = l.begin(); it != l.end(); ++it) {
 	QFileInfo fi(Option::fixPathToLocalOS((*it)));
 	QString dirName;
-	if ( dir.isEmpty() )
-	    dirName = Option::fixPathToTargetOS(fi.dirPath(), FALSE) + Option::dir_sep;
-	else
+	if ( dir.isEmpty() ) {
+	    QString fName = Option::fixPathToTargetOS((*it), FALSE);
+	    int dl = fName.findRev(Option::dir_sep);
+	    if(dl != -1)
+		dirName = fName.left(dl + 1);
+	} else {
 	    dirName = dir;
+	}
 
 	file = dirName + fi.baseName(TRUE) + Option::obj_ext;
 	fileFixify(file);
@@ -1615,6 +1709,8 @@ MakefileGenerator::writeMakeQmake(QTextStream &t)
 		fileFixify(s);
 		t << s << " ";
 	    }
+	    if(!specdir().isEmpty()) 
+		t << specdir() << "/qmake.conf" << " ";
 	    t << project->variables()["QMAKE_INTERNAL_INCLUDED_FILES"].join(" \\\n\t\t") << "\n\t"
 	      << qmake <<endl;
 	}
@@ -1808,3 +1904,5 @@ MakefileGenerator::create(QMakeProject *proj)
     }
     return mkfile;
 }
+
+
