@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qdrawutil.cpp   3.0.2   edited Oct 30 15:00 $
+** $Id:  qt/qdrawutil.cpp   3.0.3   edited Mar 3 21:35 $
 **
 ** Implementation of draw utilities
 **
@@ -899,7 +899,7 @@ static void qDrawMotifArrow( QPainter *p, Qt::ArrowType type, bool down,
     QWMatrix wxm = p->worldMatrix();
 #endif
     QPen     pen( Qt::NoPen );
-    QBrush brush = g.brush( QColorGroup::Button );
+    const QBrush &brush = g.brush( QColorGroup::Button );
 
     p->setPen( pen );
     p->setBrush( brush );
