@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qrichtext_p.cpp   3.0.2   edited Jan 23 16:57 $
+** $Id:  qt/qrichtext_p.cpp   3.0.3   edited Mar 15 17:11 $
 **
 ** Implementation of the internal Qt classes dealing with rich text
 **
@@ -44,11 +44,11 @@ QTextCommand::Commands QTextCommand::type() const { return Invalid; }
 
 
 QTextCustomItem::~QTextCustomItem() {}
-void QTextCustomItem::setPainter( QPainter*, bool adjust ){ if ( adjust ) width = 0; }
+void QTextCustomItem::adjustToPainter( QPainter* p){ if ( p ) width = 0; }
 QTextCustomItem::Placement QTextCustomItem::placement() const { return PlaceInline; }
 
 bool QTextCustomItem::ownLine() const { return FALSE; }
-void QTextCustomItem::resize( QPainter*, int nwidth ){ width = nwidth; }
+void QTextCustomItem::resize( int nwidth ){ width = nwidth; }
 void QTextCustomItem::invalidate() {}
 
 bool QTextCustomItem::isNested() const { return FALSE; }
@@ -138,18 +138,23 @@ void QTextDocument::setSelectionStart( int id, QTextCursor *cursor )
 
 QTextParag *QTextDocument::paragAt( int i ) const
 {
-    QTextParag *s = fParag;
-    while ( s ) {
-	if ( s->paragId() == i )
-	    return s;
-	s = s->next();
-    }
-    return 0;
+    QTextParag* p = curParag;
+    if ( !p || p->paragId() > i )
+	p = fParag;
+    while ( p && p->paragId() != i )
+	p = p->next();
+    ((QTextDocument*)this)->curParag = p;
+    return p;
+}
+
+
+QTextFormat::~QTextFormat()
+{
 }
 
 QTextFormat::QTextFormat()
     : fm( QFontMetrics( fn ) ), linkColor( TRUE ), logicalFontSize( 3 ), stdPointSize( qApp->font().pointSize() ),
-      painter( 0 ), different( NoFlags )
+      different( NoFlags )
 {
     ref = 0;
     missp = FALSE;
@@ -159,7 +164,7 @@ QTextFormat::QTextFormat()
 
 QTextFormat::QTextFormat( const QStyleSheetItem *style )
     : fm( QFontMetrics( fn ) ), linkColor( TRUE ), logicalFontSize( 3 ), stdPointSize( qApp->font().pointSize() ),
-      painter( 0 ), different( NoFlags )
+      different( NoFlags )
 {
     ref = 0;
     this->style = style->name();
@@ -175,8 +180,8 @@ QTextFormat::QTextFormat( const QStyleSheetItem *style )
     fm = QFontMetrics( fn );
     leftBearing = fm.minLeftBearing();
     rightBearing = fm.minRightBearing();
-    hei = fm.height();
-    asc = fm.ascent();
+    hei = fm.lineSpacing();
+    asc = fm.ascent() + (fm.leading()+1)/2;
     dsc = fm.descent();
     missp = FALSE;
     ha = AlignNormal;
@@ -188,15 +193,14 @@ QTextFormat::QTextFormat( const QStyleSheetItem *style )
 
 QTextFormat::QTextFormat( const QFont &f, const QColor &c, QTextFormatCollection *parent )
     : fn( f ), col( c ), fm( QFontMetrics( f ) ), linkColor( TRUE ),
-      logicalFontSize( 3 ), stdPointSize( f.pointSize() ), painter( 0 ),
-      different( NoFlags )
+      logicalFontSize( 3 ), stdPointSize( f.pointSize() ), different( NoFlags )
 {
     ref = 0;
     collection = parent;
     leftBearing = fm.minLeftBearing();
     rightBearing = fm.minRightBearing();
-    hei = fm.height();
-    asc = fm.ascent();
+    hei = fm.lineSpacing();
+    asc = fm.ascent() + (fm.leading()+1)/2;
     dsc = fm.descent();
     missp = FALSE;
     ha = AlignNormal;
@@ -213,7 +217,6 @@ QTextFormat::QTextFormat( const QTextFormat &f )
     collection = 0;
     fn = f.fn;
     col = f.col;
-    painter = f.painter;
     leftBearing = f.leftBearing;
     rightBearing = f.rightBearing;
     memset( widths, 0, 256 );
@@ -225,8 +228,6 @@ QTextFormat::QTextFormat( const QTextFormat &f )
     missp = f.missp;
     ha = f.ha;
     k = f.k;
-    anchor_name = f.anchor_name;
-    anchor_href = f.anchor_href;
     linkColor = f.linkColor;
     style = f.style;
     different = f.different;
@@ -251,8 +252,6 @@ QTextFormat& QTextFormat::operator=( const QTextFormat &f )
     missp = f.missp;
     ha = f.ha;
     k = f.k;
-    anchor_name = f.anchor_name;
-    anchor_href = f.anchor_href;
     linkColor = f.linkColor;
     style = f.style;
     different = f.different;
@@ -265,52 +264,74 @@ void QTextFormat::update()
     fm = QFontMetrics( fn );
     leftBearing = fm.minLeftBearing();
     rightBearing = fm.minRightBearing();
-    hei = fm.height();
-    asc = fm.ascent();
+    hei = fm.lineSpacing();
+    asc = fm.ascent() + (fm.leading()+1)/2;
     dsc = fm.descent();
     memset( widths, 0, 256 );
     generateKey();
     updateStyleFlags();
 }
 
+
+QPainter* QTextFormat::pntr = 0;
+
+void QTextFormat::setPainter( QPainter *p )
+{
+    pntr = p;
+}
+
+QPainter*  QTextFormat::painter()
+{
+    return pntr;
+}
+
+
 int QTextFormat::minLeftBearing() const
 {
-    if ( !painter || !painter->isActive() )
+    if ( !pntr || !pntr->isActive() )
 	return leftBearing;
-    painter->setFont( fn );
-    return painter->fontMetrics().minLeftBearing();
+    pntr->setFont( fn );
+    return pntr->fontMetrics().minLeftBearing();
 }
 
 int QTextFormat::minRightBearing() const
 {
-    if ( !painter || !painter->isActive() )
+    if ( !pntr || !pntr->isActive() )
 	return rightBearing;
-    painter->setFont( fn );
-    return painter->fontMetrics().minRightBearing();
+    pntr->setFont( fn );
+    return pntr->fontMetrics().minRightBearing();
 }
 
 int QTextFormat::height() const
 {
-    if ( !painter || !painter->isActive() )
+    if ( !pntr || !pntr->isActive() )
 	return hei;
-    painter->setFont( fn );
-    return painter->fontMetrics().height();
+    pntr->setFont( fn );
+    return pntr->fontMetrics().lineSpacing();
 }
 
 int QTextFormat::ascent() const
 {
-    if ( !painter || !painter->isActive() )
+    if ( !pntr || !pntr->isActive() )
 	return asc;
-    painter->setFont( fn );
-    return painter->fontMetrics().ascent();
+    pntr->setFont( fn );
+    return pntr->fontMetrics().ascent() + (pntr->fontMetrics().leading()+1)/2;
 }
 
 int QTextFormat::descent() const
 {
-    if ( !painter || !painter->isActive() )
+    if ( !pntr || !pntr->isActive() )
 	return dsc;
-    painter->setFont( fn );
-    return painter->fontMetrics().descent();
+    pntr->setFont( fn );
+    return pntr->fontMetrics().descent();
+}
+
+int QTextFormat::leading() const
+{
+    if ( !pntr || !pntr->isActive() )
+	return fm.leading();
+    pntr->setFont( fn );
+    return pntr->fontMetrics().leading();
 }
 
 void QTextFormat::generateKey()
@@ -325,13 +346,10 @@ void QTextFormat::generateKey()
        << (uint)col.rgb() << "/"
        << fn.family() << "/"
        << (int)isMisspelled() << "/"
-       << anchor_href << "/"
-       << anchor_name << "/"
        << (int)vAlign();
 }
 
-QString QTextFormat::getKey( const QFont &fn, const QColor &col, bool misspelled,
-				    const QString &lhref, const QString &lnm, VerticalAlignment a )
+QString QTextFormat::getKey( const QFont &fn, const QColor &col, bool misspelled, VerticalAlignment a )
 {
     QString k;
     QTextOStream ts( &k );
@@ -343,8 +361,6 @@ QString QTextFormat::getKey( const QFont &fn, const QColor &col, bool misspelled
        << (uint)col.rgb() << "/"
        << fn.family() << "/"
        << (int)misspelled << "/"
-       << lhref << "/"
-       << lnm << "/"
        << (int)a;
     return k;
 }
@@ -420,27 +436,6 @@ QString QTextString::toString() const
     return toString( data );
 }
 
-QString QTextString::toReverseString() const
-{
-    QString s;
-    int l = length();
-    s.setUnicode(0, l);
-    QTextStringChar *c = data.data() + (l-1);
-    QChar *uc = (QChar *)s.unicode();
-    while ( l-- ) {
-	*uc = c->c;
-	// ### workaround so that non-breaking whitespaces are drawn
-	// properly, actually this should be fixed in QFont somewhere
-	if ( *uc == (char)0xa0 )
-	    *uc = 0x20;
-	uc++;
-	c--;
-    }
-
-    return s;
-}
-
-
 void QTextParag::setSelection( int id, int start, int end )
 {
     QMap<int, QTextParagSelection>::ConstIterator it = selections().find( id );
@@ -696,12 +691,8 @@ QTextStringChar::~QTextStringChar()
 {
     if ( format() )
 	format()->removeRef();
-    switch ( type ) {
-	case Custom:
-	    delete d.custom; break;
-	default:
-	    break;
-    }
+    if ( type ) // not Regular
+	delete d.custom;
 }
 
 QTextParagPseudoDocument::QTextParagPseudoDocument():pFormatter(0),commandHistory(0), minw(0),wused(0){}
