@@ -1,5 +1,5 @@
 /****************************************************************************
-** $Id:  qt/qspinbox.cpp   3.0.2   edited Dec 18 16:54 $
+** $Id:  qt/qspinbox.cpp   3.0.3   edited Mar 15 12:08 $
 **
 ** Implementation of QSpinBox widget class
 **
@@ -48,6 +48,7 @@
 #include "qpixmapcache.h"
 #include "qapplication.h"
 #include "qtimer.h"
+#include "qstyle.h"
 #if defined(QT_ACCESSIBILITY_SUPPORT)
 #include "qaccessible.h"
 #endif
@@ -60,9 +61,59 @@ public:
     uint selreq 	: 1;
 };
 
+class QSpinBoxValidator : public QIntValidator
+{
+public:
+    QSpinBoxValidator( QSpinBox *sb, const char *name )
+	: QIntValidator( sb, name ), spinBox( sb ) { }
+
+    virtual State validate( QString& str, int& pos ) const;
+
+private:
+    QSpinBox *spinBox;
+};
+
+QValidator::State QSpinBoxValidator::validate( QString& str, int& pos ) const
+{
+    QString pref = spinBox->prefix();
+    QString suff = spinBox->suffix();
+    uint overhead = pref.length() + suff.length();
+    State state = Invalid;
+
+    ((QIntValidator *) this)->setRange( spinBox->minValue(),
+					spinBox->maxValue() );
+    if ( overhead == 0 ) {
+	state = QIntValidator::validate( str, pos );
+    } else {
+	if ( str.length() >= overhead && str.startsWith(pref) &&
+	     str.endsWith(suff) ) {
+	    QString core = str.mid( pref.length(), str.length() - overhead );
+	    int corePos = pos - pref.length();
+	    state = QIntValidator::validate( core, corePos );
+	    pos = corePos + pref.length();
+	    str.replace( pref.length(), str.length() - overhead, core );
+	} else {
+	    state = QIntValidator::validate( str, pos );
+	    if ( state == Invalid ) {
+		// stripWhiteSpace(), cf. QSpinBox::interpretText()
+		QString special = spinBox->specialValueText().stripWhiteSpace();
+		QString candidate = str.stripWhiteSpace();
+
+		if ( special.startsWith(candidate) ) {
+		    if ( candidate.length() == special.length() ) {
+			state = Acceptable;
+		    } else {
+			state = Intermediate;
+		    }
+		}
+	    }
+	}
+    }
+    return state;
+}
 
 /*!
-  \class QSpinBox qspinbox.h
+  \class QSpinBox
 
   \brief The QSpinBox class provides a spin box widget (spin button).
 
@@ -133,7 +184,7 @@ public:
 
 	QString mapValueToText( int value )
 	{
-	    if ( value == -1 ) // Special case
+	    if ( value == -1 ) // special case
 		return QString( "Auto" );
 
 	    return QString( "%1.%2" ) // 0.0 to 10.0
@@ -142,7 +193,7 @@ public:
 
 	int mapTextToValue( bool *ok )
 	{
-	    if ( text() == "Auto" ) // Special case
+	    if ( text() == "Auto" ) // special case
 		return -1;
 
 	    return (int) ( 10 * text().toFloat() ); // 0 to 100
@@ -204,7 +255,7 @@ void QSpinBox::initSpinBox()
     edited = FALSE;
     d->selreq = FALSE;
 
-    validate = new QIntValidator( minValue(), maxValue(), this, "validator" );
+    validate = new QSpinBoxValidator( this, "validator" );
     vi = new QLineEdit( this, "qt_spinbox_edit" );
     d->controls->setEditWidget( vi );
     vi->setValidator( validate );
@@ -251,7 +302,7 @@ QString QSpinBox::text() const
   \property QSpinBox::cleanText
 
   \brief the text of the spin box with any prefix() or suffix() and
-  with any whitespace at the start and end removed.
+  any whitespace at the start and end removed.
 
   \sa text, prefix, suffix
 */
@@ -432,7 +483,7 @@ QSize QSpinBox::sizeHint() const
 {
     constPolish();
     QSize sz = vi->sizeHint();
-    int h = QMAX( sz.height(), 20 );
+    int h = sz.height();
     QFontMetrics fm( font() );
     int w = 35;
     int wx = fm.width( ' ' )*2;
@@ -446,7 +497,7 @@ QSize QSpinBox::sizeHint() const
 	w = QMAX( w, fm.width( s ) + wx );
     }
 
-    return QSize( w + d->controls->downRect().width(), h ).expandedTo( QApplication::globalStrut() );
+    return QSize( w + d->controls->downRect().width(), h + style().defaultFrameWidth() * 2 ).expandedTo( QApplication::globalStrut() );
 }
 
 
@@ -651,7 +702,7 @@ void QSpinBox::wheelEvent( QWheelEvent * e )
 
 void QSpinBox::valueChange()
 {
-    d->selreq = TRUE;
+    d->selreq = hasFocus();
     updateDisplay();
     d->selreq = FALSE;
     emit valueChanged( value() );
@@ -670,8 +721,6 @@ void QSpinBox::valueChange()
 
 void QSpinBox::rangeChange()
 {
-    if ( validate->inherits( "QIntValidator" ) )
-	((QIntValidator*)validate)->setRange( minValue(), maxValue() );
     updateDisplay();
 }
 
@@ -713,14 +762,20 @@ const QValidator * QSpinBox::validator() const
 */
 void QSpinBox::updateDisplay()
 {
+    vi->setUpdatesEnabled( FALSE );
     vi->setText( currentValueText() );
-    if ( d->selreq && isVisible() )
+    if ( d->selreq && isVisible() && ( hasFocus() || vi->hasFocus() ) ) {
 	selectAll();
-    vi->repaint( FALSE ); // we want an immediate repaint, might be that a widget connected to the value changed does some longer stuff which would result in a bad feedback of the spinbox
+    } else {
+	if ( !suffix().isEmpty() && vi->text().endsWith(suffix()) )
+	     vi->setCursorPosition( vi->text().length() - suffix().length() );
+    }
+    vi->setUpdatesEnabled( TRUE );
+    vi->repaint( FALSE ); // immediate repaint needed for some reason
     edited = FALSE;
 
-    bool upEnabled = isEnabled() && (wrapping() || value() < maxValue());
-    bool downEnabled = isEnabled() && (wrapping() || value() > minValue());
+    bool upEnabled = isEnabled() && ( wrapping() || value() < maxValue() );
+    bool downEnabled = isEnabled() && ( wrapping() || value() > minValue() );
 
     d->controls->setUpEnabled( upEnabled );
     d->controls->setDownEnabled( downEnabled );
@@ -747,8 +802,8 @@ void QSpinBox::interpretText()
     bool done = FALSE;
     int newVal = 0;
     if ( !specialValueText().isEmpty() ) {
-	QString s = QString(text()).stripWhiteSpace();
-	QString t = QString(specialValueText()).stripWhiteSpace();
+	QString s = text().stripWhiteSpace();
+	QString t = specialValueText().stripWhiteSpace();
 	if ( s == t ) {
 	    newVal = minValue();
 	    done = TRUE;
@@ -758,7 +813,7 @@ void QSpinBox::interpretText()
 	newVal = mapTextToValue( &ok );
     if ( ok )
 	setValue( newVal );
-    updateDisplay();		// Sometimes redundant
+    updateDisplay(); // sometimes redundant
 }
 
 
@@ -798,12 +853,10 @@ QLineEdit* QSpinBox::editor() const
 
 void QSpinBox::textChanged()
 {
-    edited = TRUE;	// This flag is cleared in updateDisplay()
+    edited = TRUE; // this flag is cleared in updateDisplay()
 }
 
 
-
-
 /*!  This virtual function is used by the spin box whenever it needs
   to display value \a v.  The default implementation returns a string
   containing \a v printed in the standard way. Reimplementations may
@@ -999,11 +1052,16 @@ void QSpinBox::setLineStep( int i )
     setSteps( i, pageStep() );
 }
 
-/*! Selects all the text in the editor of the spinbox */
+/*! Selects all the text in the editor of the spinbox. */
 
 void QSpinBox::selectAll()
 {
-    editor()->selectAll();
+    int overhead = prefix().length() + suffix().length();
+    if ( !overhead || currentValueText() == specialValueText() ) {
+	vi->selectAll();
+    } else {
+	vi->setSelection( prefix().length(), vi->text().length() - overhead );
+    }
 }
 
 #endif
